from azure.storage.blob import BlobServiceClient
from azure.core.exceptions import ResourceNotFoundError
from fastapi import UploadFile
from uuid import uuid4
import logging


def upload_to_azure_blob(blob: UploadFile, container_name: str = "file-uploads"):
    file_name = f"{uuid4()}/{blob.filename}"
    # Create a BlobServiceClient object from the connection string
    # TODO: Move connection string to environment variable
    blob_service_client = BlobServiceClient.from_connection_string(
        "DefaultEndpointsProtocol=https;AccountName=metricblob;AccountKey=RnWKHSUxYsL9ex0pfodN74ObvgbwYCENVtRhFVYx5+JYLCIy0/VSGpXEicctF3OFJVMWJCCCwyZM+AStgszDPA==;EndpointSuffix=core.windows.net"
    )

    # Create a ContainerClient object for the specified container
    container_client = blob_service_client.get_container_client(container_name)

    # Create a BlobClient object for the specified blob
    blob_client = container_client.get_blob_client(file_name)

    # Upload the file to the blob
    with blob.file as data:
        blob_client.upload_blob(data)

    return {"file_name": file_name, "file_url": blob_client.url}


def upload_file_to_azure_blob(
    file: UploadFile, file_path: str, container_name: str = "media"
) -> dict:
    """
    Uploads a file to Azure Blob Storage.

    :param file: The file to be uploaded (FastAPI UploadFile object).
    :param file_path: The path to the file in Azure Blob Storage.
    :param container_name: The name of the Azure Blob Storage container (default is "media").
    :return: A dictionary containing the file name and its URL if successful, or None if an error occurred.
    """
    try:
        file_name = file.filename
        # Create a BlobServiceClient object from the connection string
        blob_service_client = BlobServiceClient.from_connection_string(
            "DefaultEndpointsProtocol=https;AccountName=metricblob;AccountKey=RnWKHSUxYsL9ex0pfodN74ObvgbwYCENVtRhFVYx5+JYLCIy0/VSGpXEicctF3OFJVMWJCCCwyZM+AStgszDPA==;EndpointSuffix=core.windows.net"
        )

        # Create a ContainerClient object for the specified container
        container_client = blob_service_client.get_container_client(container_name)

        # Create a BlobClient object for the specified blob
        blob_client = container_client.get_blob_client(file_path)

        # Upload the file to the blob
        with file.file as data:
            blob_client.upload_blob(data)

        return {"file_name": file_name, "file_url": blob_client.url}
    except Exception as e:
        logging.error(
            {
                "error": f"An error occurred while uploading {file_name}",
                "details": str(e),
            }
        )
        return {"file_name": file_name, "file_url": None}


def delete_file_from_azure_blob(
    file_name: str, container_name: str = "file-uploads"
) -> dict:
    try:
        # Create a BlobServiceClient object from the connection string
        blob_service_client = BlobServiceClient.from_connection_string(
            "DefaultEndpointsProtocol=https;AccountName=metricblob;AccountKey=RnWKHSUxYsL9ex0pfodN74ObvgbwYCENVtRhFVYx5+JYLCIy0/VSGpXEicctF3OFJVMWJCCCwyZM+AStgszDPA==;EndpointSuffix=core.windows.net"
        )

        # Create a ContainerClient object for the specified container
        container_client = blob_service_client.get_container_client(container_name)

        # Create a BlobClient object for the specified blob
        blob_client = container_client.get_blob_client(file_name)

        # Delete the blob
        blob_client.delete_blob()

    except ResourceNotFoundError as e:
        logging.error({"error": f"Blob not found: {file_name}", "details": str(e)})

    except Exception as e:
        logging.error(
            {
                "error": f"An error occurred while deleting {file_name}",
                "details": str(e),
            }
        )
